# $Id: MetroMirror.pm,v 1.2.2.1 2007/08/30 17:39:33 uli Exp $
# ident "@(#)VCS:$RCSfile: MetroMirror.pm,v $	$Revision: 1.2.2.1 $"
#
# START COPYRIGHT-NOTICE: 2002-2006
#
# Copyright 2002-2006 VERITAS Software Corporation.
# All rights reserved.
#
# VERITAS, VERITAS SOFTWARE, the VERITAS logo and all other VERITAS
# product names and slogans are trademarks or registered trademarks
# of VERITAS Software Corporation in the USA and/or other countries.
# Other product names and/or slogans mentioned herein may be trade-
# marks or registered trademarks of their respective companies.
#
# END COPYRIGHT-NOTICE.
#

package MetroMirror;
use ArrayRep;
@ISA = ("ArrayRep");

use strict;
use ag_i18n_inc;

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = {};

	$self->{resname} = shift;
	my $dscli_home = shift;
	my $hmc1 = shift;
	my $hmc2 = shift;
	my $user = shift;
	my $pwfile = shift;

	$self->{limgid} = shift;
	$self->{rimgid} = shift;

	# put volids in a hash where local volid is the key and
	# corresponding remote metromirror volid will be the value.
	# remote volid (i.e value field) to be populated later when determined.
	my $num_vids = shift;
	$self->{vids} = {};

	for (my $i = 0; $i < $num_vids; $i++) {
		$self->{vids}->{$_[$i]} = "";
	}

	my $dscli = join '/', $dscli_home, "dscli";
	if ($hmc2) {
		$self->{dscli} = [ $dscli, "-hmc1", $hmc1, "-hmc2", $hmc2, "-user", $user, "-pwfile", $pwfile ];
	} else {
		$self->{dscli} = [ $dscli, "-hmc1", $hmc1, "-user", $user, "-pwfile", $pwfile ];
	}

	return bless $self, $class;
}

sub DESTROY {}
	
sub get_state {
	my @pairs = ("\n");
	my $self = shift;

	$self->{state} = "";

	my @lspprc = (@{$self->{dscli}}, "lspprc", "-fmt delim", "-hdr off", "-bnr off", "-dev", $self->{limgid}, keys %{$self->{vids}});
	open(fh, "@lspprc |") or die "Command failed:@lspprc :$!";
	while (<fh>) {
		push (@pairs, $_);
		my ($id, $state, $reason, $type, $SourceLSS, $rest) = split /,/;
		next unless ($type eq "Metro Mirror");
		my ($src_vid, $tgt_vid) = split(/:/, $id);

		# set remote metromirror volID if not already set.
		if (exists $self->{vids}->{$src_vid}) {
			$self->{vids}->{$src_vid} = $tgt_vid
				if ($self->{vids}->{$src_vid} eq "");
		} elsif (exists $self->{vids}->{$tgt_vid}) {
			$self->{vids}->{$tgt_vid} = $src_vid
				if ($self->{vids}->{$tgt_vid} eq "");
		} else {
			next;
		}

		if ($self->{state} eq "") {
			$self->{state} = $state;
		} elsif ($state ne $self->{state}) {
			# volumes are in an invalid mix of states
			VCSAG_LOG_MSG("E", "Volumes are in an invalid mix of states: @pairs", 10, "@pairs");
			$self->{state} = "Invalid";
			close(fh);
			return 1;
		}
	}

	close(fh);
	VCSAG_LOG_MSG("I", "Metro mirror pairs are @pairs", 11, "@pairs");
	return 0;
}

sub failover_required {
	my ($self) = shift;

	return 0 if ($self->{state} eq "Full Duplex");
	return 0 if ($self->{state} eq "Suspended");

	return 1 if ($self->{state} eq "Target Full Duplex");

	return 2;
}

sub failover {
	my $self = shift;

	my @failoverpprc = (@{$self->{dscli}}, "failoverpprc", "-dev", $self->{limgid}, "-remotedev", $self->{rimgid}, "-type mmir");

	while (my ($lvid, $rvid) = each %{$self->{vids}}) {
		push (@failoverpprc, (join ':', $lvid, $rvid));
	}

	VCSAG_LOG_MSG("N", "Issuing failoverpprc command: @failoverpprc", 12, "@failoverpprc");
	my $ret = system(@failoverpprc);
	if ($ret) {
		VCSAG_LOG_MSG("E", "failoverpprc command failed: $!", 13, "$!");
		return 1;
	}

	$self->create_lockfile();
	$self->cvm_import();
	return 0;
}

sub failback {
	my $self = shift;

	my @failbackpprc = (@{$self->{dscli}}, "failbackpprc", "-dev", $self->{limgid}, "-remotedev", $self->{rimgid}, "-type mmir", "-tgtread");

	while (my ($lvid, $rvid) = each %{$self->{vids}}) {
		push (@failbackpprc, (join ':', $lvid, $rvid));
	}

	VCSAG_LOG_MSG("N", "Issuing failbackpprc command: @failbackpprc", 14, "@failbackpprc");
	my $ret = system(@failbackpprc);
	if ($ret) {
		# not a problem. This can fail when original
		# primary array is not accessible
		VCSAG_LOG_MSG("I", "failbackpprc command failed: $!", 15, "$!");
		return 1;
	}
	return 0;
}

1;
